@@ -1,7 +1,10 @@
 # from __future__ import print_function, absolute_import, division
+import logging
+import math
 import multiprocessing
 import re
 import shutil
+import sys
 import types
 from pathlib import Path
 
@@ -9,7 +12,7 @@ import allesfitter
 import numpy as np
 import yaml
 from argparse import ArgumentParser
-from sherlockpipe.star.HabitabilityCalculator import HabitabilityCalculator
+from lcbuilder.star.HabitabilityCalculator import HabitabilityCalculator
 import pandas as pd
 import os
 from os import path
@@ -20,7 +23,7 @@ resources_dir = path.join(path.dirname(__file__))
 
 
 class Fitter:
-    def __init__(self, object_dir, only_initial, mcmc = False, detrend = False):
+    def __init__(self, object_dir, only_initial, mcmc=False, detrend=None):
         self.args = types.SimpleNamespace()
         self.args.noshow = True
         self.args.north = False
@@ -42,6 +45,7 @@ class Fitter:
 
     def fit(self, candidate_df, star_df, cpus, allesfit_dir):
         candidate_row = candidate_df.iloc[0]
+        logging.info("Preparing fit files")
         sherlock_star_file = self.object_dir + "/params_star.csv"
         star_file = allesfit_dir + "/params_star.csv"
         params_file = allesfit_dir + "/params.csv"
@@ -60,13 +64,18 @@ class Fitter:
             fit_width = float(candidate_row["duration"]) / 60 / 24 * 7
         # TODO replace sherlock properties from allesfitter files
         # TODO only use params_star when the star mass or radius was not assumed
+        logging.info("Preparing fit properties")
         with open(settings_file, 'r+') as f:
             text = f.read()
             text = re.sub('\\${sherlock:cores}', str(cpus), text)
             text = re.sub('\\${sherlock:fit_width}', str(fit_width), text)
             text = re.sub('\\${sherlock:name}', str(candidate_row["name"]), text)
-            detrend_param = "baseline_flux_lc,hybrid_offset"
-            detrend_param = detrend_param if self.detrend else "#" + detrend_param
+            if self.detrend == 'hybrid_spline':
+                detrend_param = "baseline_flux_lc,hybrid_spline"
+            elif self.detrend == 'gp':
+                detrend_param = 'baseline_flux_lc,sample_GP_Matern32'
+            else:
+                detrend_param = ''
             text = re.sub('\\${sherlock:detrend}', detrend_param, text)
             f.seek(0)
             f.write(text)
@@ -74,31 +83,79 @@ class Fitter:
         with open(params_file, 'r+') as f:
             text = f.read()
             text = re.sub('\\${sherlock:t0}', str(candidate_row["t0"]), text)
+            text = re.sub('\\${sherlock:t0_min}', str(candidate_row["t0"] - 0.02), text)
+            text = re.sub('\\${sherlock:t0_max}', str(candidate_row["t0"] + 0.02), text)
             text = re.sub('\\${sherlock:period}', str(candidate_row["period"]), text)
-            rp_rs = candidate_row["rp_rs"] if candidate_row["rp_rs"] != "-" else 0.1
-            text = re.sub('\\${sherlock:rp_rs}', str(rp_rs), text)
-            sum_rp_rs_a = (candidate_row["rp_rs"] + star_df.iloc[0]['R_star']) / candidate_row["a"] * 0.00465047 \
-                if candidate_row["rp_rs"] != "-" else 0.2
-            text = re.sub('\\${sherlock:sum_rp_rs_a}', str(sum_rp_rs_a), text)
-            text = re.sub('\\${sherlock:name}', str(candidate_row["name"]), text)
-            if os.path.exists(sherlock_star_file) and os.path.isfile(sherlock_star_file):
-                text = re.sub('\\${sherlock:ld_a}', str(star_df.iloc[0]["ld_a"]) + ",0", text)
-                text = re.sub('\\${sherlock:ld_b}', str(star_df.iloc[0]["ld_b"]) + ",0", text)
+            text = re.sub('\\${sherlock:period_min}', str(candidate_row["period"] - candidate_row["per_err"]), text)
+            text = re.sub('\\${sherlock:period_max}', str(candidate_row["period"] + candidate_row["per_err"]), text)
+            if self.detrend == 'gp':
+                baseline_params = 'baseline_gp_matern32_lnsigma_flux_lc,0.0,1,uniform -15.0 15.0,$\mathrm{gp: \ln{\sigma} (lc)}$,\n' + \
+                    'baseline_gp_matern32_lnrho_flux_lc,0.0,1,uniform -15.0 15.0,$\mathrm{gp: \ln{\rho} (lc)}$,'
             else:
-                text = re.sub('\\${sherlock:ld_a}', "0.5,1", text)
-                text = re.sub('\\${sherlock:ld_b}', "0.5,1", text)
+                baseline_params = ""
+            text = re.sub('\\${sherlock:baseline_params}', baseline_params, text)
+            rp_rs = candidate_row["rp_rs"] if candidate_row["rp_rs"] != "-" else 0.1
+            depth = candidate_row["depth"] / 1000
+            # TODO calculate depth error in SHERLOCK maybe given the std deviation from the depths or even using the residuals
+            depth_err = depth * 0.2
+            rp_rs_err = 0.5 / math.sqrt(depth) * depth_err
+            text = re.sub('\\${sherlock:rp_rs}', str(rp_rs), text)
+            rp_rs_min = rp_rs - 2 * rp_rs_err
+            rp_rs_min = rp_rs_min if rp_rs_min > 0 else 0.0000001
+            rp_rs_max = rp_rs + 2 * rp_rs_err
+            rp_rs_max = rp_rs_max if rp_rs_max < 1 else 0.9999
+            text = re.sub('\\${sherlock:rp_rs_min}', str(rp_rs_min), text)
+            text = re.sub('\\${sherlock:rp_rs_max}', str(rp_rs_max), text)
+            G = 6.674e-11
+            mstar_kg = star_df.iloc[0]["M_star"] * 2e30
+            mstar_kg_lerr = star_df.iloc[0]["M_star_lerr"] * 2e30
+            mstar_kg_uerr = star_df.iloc[0]["M_star_uerr"] * 2e30
+            rstar_au = star_df.iloc[0]['R_star'] * 0.00465047
+            rstar_lerr_au = star_df.iloc[0]["R_star_lerr"] * 0.00465047
+            rstar_uerr_au = star_df.iloc[0]["R_star_uerr"] * 0.00465047
+            per = candidate_row["period"] * 24 * 3600
+            per_err = candidate_row["per_err"] * 24 * 3600
+            a = (G * mstar_kg * per ** 2 / 4. / (np.pi ** 2)) ** (1. / 3.)
+            a_au = a / 1.496e11
+            sum_rp_rs_a = (np.sqrt(depth) + 1.) * rstar_au / a_au
+            sum_rp_rs_a_lerr = np.sqrt(
+                np.square(0.5 * depth_err / depth) + np.square(rstar_lerr_au / rstar_au) + np.square(
+                    2 * per_err / 3. / per) + np.square(mstar_kg_lerr / 3. / mstar_kg)) * sum_rp_rs_a
+            sum_rp_rs_a_uerr = np.sqrt(
+                np.square(0.5 * depth_err / depth) + np.square(rstar_uerr_au / rstar_au) + np.square(
+                    2 * per_err / 3. / per) + np.square(mstar_kg_uerr / 3. / mstar_kg)) * sum_rp_rs_a
+            sum_rp_rs_a_min = sum_rp_rs_a - 2 * sum_rp_rs_a_lerr
+            sum_rp_rs_a_max = sum_rp_rs_a + 2 * sum_rp_rs_a_uerr
+            sum_rp_rs_a_min = sum_rp_rs_a_min if sum_rp_rs_a_min > 0 else 0.0000001
+            text = re.sub('\\${sherlock:sum_rp_rs_a}', str(sum_rp_rs_a), text)
+            text = re.sub('\\${sherlock:sum_rp_rs_a_min}', str(sum_rp_rs_a_min), text)
+            text = re.sub('\\${sherlock:sum_rp_rs_a_max}', str(sum_rp_rs_a_max), text)
+            text = re.sub('\\${sherlock:name}', str(candidate_row["name"]), text)
+            # TODO this check is wrong and might need to check whether ld_a and ld_b exist within the dataframe
+            if os.path.exists(sherlock_star_file) and os.path.isfile(sherlock_star_file):
+                text = re.sub('\\${sherlock:ld_a}', str(star_df.iloc[0]["ld_a"]), text)
+                text = re.sub('\\${sherlock:ld_b}', str(star_df.iloc[0]["ld_b"]), text)
+            else:
+                text = re.sub('\\${sherlock:ld_a}', "0.5", text)
+                text = re.sub('\\${sherlock:ld_b}', "0.5", text)
             f.seek(0)
             f.write(text)
             f.truncate()
+
+        logging.info("Running initial guess")
         allesfitter.show_initial_guess(allesfit_dir)
         self.custom_plot(candidate_row["name"], candidate_row["period"], fit_width, allesfit_dir, "initial_guess")
         if not self.only_initial:
+            logging.info("Preparing bayesian fit")
             if not self.mcmc:
+                logging.info("Running dynamic nested sampling")
                 allesfitter.ns_fit(allesfit_dir)
                 allesfitter.ns_output(allesfit_dir)
             elif self.mcmc:
+                logging.info("Running MCMC")
                 allesfitter.mcmc_fit(allesfit_dir)
                 allesfitter.mcmc_output(allesfit_dir)
+            logging.info("Generating custom plots")
             self.custom_plot(candidate_row["name"], candidate_row["period"], fit_width, allesfit_dir, "posterior")
 
     def custom_plot(self, name, period, fit_width, allesfit_dir, mode="posterior"):
@@ -142,11 +199,27 @@ if __name__ == '__main__':
     ap.set_defaults(only_initial=False)
     ap.add_argument('--cpus', type=int, default=None, help="The number of CPU cores to be used.", required=False)
     ap.add_argument('--mcmc', dest='mcmc', action='store_true', help="Whether to run using mcmc or ns. Default is ns.")
-    ap.add_argument('--detrend', dest='detrend', action='store_true', help="Whether to execute detrending in the "
-                                                                            "allesfitter runs.")
+    ap.add_argument('--detrend', dest='detrend', default="hybrid_spline", help="Type of detrending to be used", required=False,
+                    choices=['no', 'gp'])
     ap.add_argument('--properties', help="The YAML file to be used as input.", required=False)
     args = ap.parse_args()
     fitter = Fitter(args.object_dir, args.only_initial, args.mcmc, args.detrend)
+    file_dir = fitter.object_dir + "/fit.log"
+    if os.path.exists(file_dir):
+        os.remove(file_dir)
+    formatter = logging.Formatter('%(message)s')
+    logger = logging.getLogger()
+    while len(logger.handlers) > 0:
+        logger.handlers.pop()
+    logger.setLevel(logging.INFO)
+    handler = logging.StreamHandler(sys.stdout)
+    handler.setLevel(logging.INFO)
+    handler.setFormatter(formatter)
+    logger.addHandler(handler)
+    handler = logging.FileHandler(file_dir)
+    handler.setLevel(logging.INFO)
+    handler.setFormatter(formatter)
+    logger.addHandler(handler)
     index = 0
     fitting_dir = fitter.data_dir + "/fit_" + str(index)
     while os.path.exists(fitting_dir) or os.path.isdir(fitting_dir):
@@ -198,5 +271,5 @@ if __name__ == '__main__':
             cpus = multiprocessing.cpu_count() - 1
         else:
             cpus = args.cpus
-        print("Selected signal number " + str(candidate_selection))
+        logging.info("Selected signal number " + str(candidate_selection))
     fitter.fit(candidate, star_df, cpus, fitting_dir)
